module Main where
import Data.Bits
import Data.Char
import Data.List
import Data.Maybe
{-
 - https://kmb.cybber.ru/crypto/frequency_analysis_substitution/main.html
 - Здесь будет конвертатор в base32/64
 - Шифр цезаря
 - Шифр простой замены
 - Шифр Атбаша
 - Шифр транспонирования
 -}

-- Шифр Цезаря. Сдвиг на N символов в X сторону.
-- Input, Direction(Y=right, N=left), Number
-- Algo: если сдвигаем вправо, то к алфавиту 
-- [*начало+сдвиг*..'z'] прибавляем алфавит ['a'..*конец-сдвиг*]
-- если сдвиг влево, то [] : []
caesarEnc :: String -> Bool -> Int -> String
caesarEnc input direction numb = do
  let alp = ['a'..'z']
  if direction
  then encEr input alp (concat [(drop numb alp), (take numb alp)])
  else "NaN"

-- Подменялка алфавита. Input, AlphabetOrig, AlphabetNew
encEr :: String -> String -> String -> String
encEr input alp encAlp = map (\x -> encAlp !! (fromJust (elemIndex x alp))) input

-- Шифр Атбаша. Инверсия алфавита.
-- a = e, r=d и т.д.
atbashEnc :: String -> String
atbashEnc input = do
  let alp = ['a'..'z']
  let encAlp = reverse alp
  map (\x -> encAlp !! (fromJust (elemIndex x alp))) input
-- Декодер
atbashDec :: String -> String
atbashDec input = do
  let alp = ['a'..'z']
  let encAlp = reverse alp
  map (\x -> alp !! (fromJust (elemIndex x encAlp))) input

-- Простейший шифр, просто сдвигает алфавит на 1
-- a = b, b = c, ...
simpliestEnc :: String -> String
simpliestEnc input = 
 map (\x -> chr(ord(x) + 1)) input 

simpliestDec :: String -> String
simpliestDec input =
  map (\x -> chr(ord(x) - 1)) input

main :: IO ()
main = do
  putStrLn "Простейшее шифрование, просто сдвигаем алфавит:"
  print (simpliestEnc "abcdefz")
  print . simpliestDec $ "bcdefg{"
  putStrLn "Шифр Атбаша, переворот алфавита:"
  print . atbashEnc $ "test"
  print . atbashDec $ ( atbashEnc "test" )
  putStrLn "Шифт Цезаря (вправо на 3):"
  print (caesarEnc "test" True 3)
